==> lori_sdk-0.7.3.tar.gz: lib/data/models/product/product.dart <==
import 'dart:convert';

import 'package:json_annotation/json_annotation.dart';
import 'package:lori_sdk/data/lori_api.dart';
import 'package:lori_sdk/data/models/product/featuredCharacteristic.dart';
import 'package:lori_sdk/data/models/product/productImage.dart';
import 'package:lori_sdk/data/models/product/skuAttribute.dart';
import 'package:lori_sdk/data/models/key_value.dart';

part 'product.g.dart';

const ORDER_RELEVANCE = "Relevance";
const ORDER_HIGHER = "HigherPrice";
const ORDER_LOWEST = "LowestPrice";
const ORDER_FAVORED = "MostFavored";
const ORDER_SELLERS = "BestSellers";

final String tableName = 'product';
final String columnId = 'id';
final String columnName = 'name';
final String columnCode = 'code';
final String columnAuxCode = 'aux_code';
final String columnReferenceCode = 'reference_code';
final String columnDescription = 'description';
final String columnAdditionalInfo = 'additional_info';
final String columnImageUrl = 'image_url';

///OLD PRICE PROPERTIES
final String columnEcommerceID = 'ecommerce_id';
final String columnEcommercePrice = 'ecommerce_price';
final String columnEcommercestock = 'ecommerce_stock';
final String columnKioskId = 'kiosk_id';
final String columnKioskPrice = 'kiosk_price';
final String columnKioskstock = 'kiosk_stock';
final String columnDefaultPrice = 'default_price';
final String columnSingleOldPrice = 'single_old_price';
final String columnSinglePrice = 'single_price';
final String columnMaxKioskInstallments = 'max_kiosk_installments';
final String columnKioskInstallmentPrice = 'kiosk_installment_price';
final String columnMaxEcommerceInstallments = 'max_ecommerce_installments';
final String columnEcommerceInstallmentPrice = 'ecommerce_installment_price';
final String columnMaxsingleInstallments = 'max_single_installments';
final String columnSingleInstallmentPrice = 'single_installment_price';
final String columnIssinglePrice = 'is_single_price';
final String columnFeaturedCharacteristics = 'featured_characteristics';
///OLD PRICE PROPERTIES

///NEW PRICE PROPERTIES
final String columnPrice = 'price';
final String columnOldPrice = 'old_price';
final String columnMaxInstallments = 'max_installments';
final String columnInstallmentPrice = 'installment_price';

///OLD PRICE PROPERTIES

final String columnIsFavorite = 'is_favorite';
final String columnFavoriteCount = 'favorite_count';
final String columnIsNew = 'is_new';
final String columnIsAvailable = 'is_available';
final String columnLastTimestamp = 'lastTimestamp';

@JsonSerializable()
class Product {
  Product();

  String id;
  String name;
  String code;
  String auxCode;
  String complementaryName;
  String referenceCode;
  String description;
  String additionalInfo;
  String imageUrl;
  String additionalInformation;

  ///
  /// region [START] Prices variables for LEGACY API
  ///

  String ecommerceId;
  double ecommercePrice;
  int ecommerceStock;

  String kioskId;
  double kioskPrice;
  int kioskStock;

  double defaultPrice;
  double singleOldPrice;
  double singlePrice;
  double unitPrice;

  int maxKioskInstallments;
  double kioskInstallmentPrice;
  int maxEcommerceInstallments;
  double ecommerceInstallmentPrice;
  int maxSingleInstallments;
  double singleInstallmentPrice;
  bool isSinglePrice;

  ///
  /// [END] Prices variables for LEGACY API
  /// endregion
  ///

  ///
  /// region [START] Prices properties for MODERN API
  ///
  ///

  double price;
  double oldPrice;
  double wholesalePrice;
  int wholesaleMinQuantity;

  int maxInstallments;
  double installmentPrice;

  ///
  /// [END] Prices properties for MODERN API

  List<Sku> skus;
  List<ProductImage> images;

  bool isFavorite;
  int favoriteCount;
  bool isNew;

  @JsonKey(defaultValue: true)
  bool isAvailable;

  int lastTimestamp;


  @JsonKey(defaultValue: false)
  bool pickupOnly;

  @JsonKey(defaultValue: [])
  List<FeaturedCharacteristic> featuredCharacteristics;

  @JsonKey(defaultValue: [])
  List<KeyValue> characteristics;

  List<KeyValue> orderCharacteristics() {
    if (this.characteristics != null && this.characteristics.isNotEmpty) {
      this.characteristics.sort((a, b) => a.order.compareTo(b.order));
      return this.characteristics;
    }
    return this.characteristics;
  }

  bool haveMessagefeaturedCharacteristics() {
    if(featuredCharacteristics != null && featuredCharacteristics.isNotEmpty) {
      return true;
    }
    return false;
  }

  double getOldPrice() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.oldPrice != null) return this.oldPrice;

    // Fallback to legacy API
    if (isSinglePrice) {
      return singleOldPrice;
    } else {
      //TODO: Add this to Kiosk/Ecommerce Price if we ever use it again
      return singleOldPrice;
    }
  }

  double getProductPrice() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.price != null && this.price > 0) return this.price;

    // Fallback to legacy API
    if (isSinglePrice)
      return singlePrice;
    else {
      if (this.kioskPrice <= this.ecommercePrice)
        return this.kioskPrice;
      else
        return this.ecommercePrice;
    }
  }

  double getInstallmentPrice() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.installmentPrice != null) return this.installmentPrice;

    // Fallback to legacy API
    if (isSinglePrice)
      return this.singleInstallmentPrice;
    else {
      if (this.kioskInstallmentPrice <= this.ecommerceInstallmentPrice)
        return this.kioskInstallmentPrice;
      else
        return this.ecommerceInstallmentPrice;
    }
  }

  int getMaxInstallments() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.maxInstallments != null) return this.maxInstallments;

    // Fallback to legacy API
    if (isSinglePrice)
      return this.maxSingleInstallments;
    else {
      if (this.maxKioskInstallments <= this.maxEcommerceInstallments)
        return this.maxKioskInstallments;
      else
        return this.maxEcommerceInstallments;
    }
  }

  String displayCode({String displayCodeName}) {
    switch (displayCodeName.toLowerCase()) {
      case "referencecode":
        //Reference Code is NOT mandatory, so we must have a fallback
        if (this.referenceCode == null) return this.code;
        return this.referenceCode.isNotEmpty ? this.referenceCode : this.code;
      case "auxcode":
        //Aux Code is NOT mandatory, so we must have a fallback
        if(this.auxCode == null) return this.code;
        return this.auxCode.isNotEmpty ? this.auxCode : this.code;
      case "code":
        return this.code;
      default:
        return this.code;
    }
  }

  List<String> get tags {
    var tags = List<String>();

    if (this.getOldPrice() != null && this.getOldPrice() > this.getProductPrice()) {
      // We have a discount at this product
      // Get delta value
      var discountValue = this.getOldPrice() - this.getProductPrice();
      // Get percentage in decimal (i.e., 0,15)
      var decimalPercentage = discountValue / this.getOldPrice();
      // Multiply by 100 to get the absolute percentage (i.e., 15)
      var percentage = decimalPercentage * 100;
      tags.add(percentage.toString().substring(0, percentage.toString().indexOf('.')) + "% OFF");
      // if(percentage is int || percentage == percentage.roundToDouble()){
      //   tags.add(percentage.toStringAsFixed(0) + "% OFF");
      // } else {
      //   percentage = (percentage * 100).truncateToDouble() / 100;
      //   tags.add(percentage.toString() + "% OFF");
      // }
    }

    return tags;
  }

  factory Product.fromJson(Map<String, dynamic> json) {
    var product = _$ProductFromJson(json);
    if (product.skus != null) {
      // Sort the skus alphabetically
      product.skus.sort((ls, rs) => ls.name.compareTo(rs.name));
    }

    return product;
  }

  Map<String, dynamic> toJson() => _$ProductToJson(this);

  Map<String, dynamic> toMap() {
    var map = <String, dynamic>{
      columnId: id,
      columnName: name,
      columnCode: code,
      columnAuxCode: auxCode,
      columnReferenceCode: referenceCode,
      columnDescription: description,
      columnAdditionalInfo: additionalInfo,
      columnImageUrl: imageUrl,

      ///OLD PRICE PROPERTIES
      columnEcommerceID: ecommerceId,
      columnEcommercePrice: ecommercePrice?.toDouble(),
      columnEcommercestock: ecommerceStock,
      columnKioskId: kioskId,
      columnKioskPrice: kioskPrice?.toDouble(),
      columnKioskstock: kioskStock,
      columnDefaultPrice: defaultPrice?.toDouble(),
      columnSingleOldPrice: singleOldPrice?.toDouble(),
      columnSinglePrice: singlePrice?.toDouble(),
      columnMaxKioskInstallments: maxKioskInstallments,
      columnKioskInstallmentPrice: kioskInstallmentPrice?.toDouble(),
      columnMaxEcommerceInstallments: maxEcommerceInstallments,
      columnEcommerceInstallmentPrice: ecommerceInstallmentPrice?.toDouble(),
      columnMaxsingleInstallments: maxSingleInstallments,
      columnSingleInstallmentPrice: singleInstallmentPrice?.toDouble(),
      columnIssinglePrice: isSinglePrice == null ? null : isSinglePrice ? 1 : 0,
      columnFeaturedCharacteristics: json.encode(featuredCharacteristics),
      ///OLD PRICE PROPERTIES
      ///
      ///NEW PRICE PROPERTIES
      columnPrice: price,
      columnOldPrice: oldPrice,
      columnMaxInstallments: maxInstallments,
      columnInstallmentPrice: installmentPrice,

      ///NEW PRICE PROPERTIES

      columnIsFavorite: isFavorite ? 1 : 0,
      columnFavoriteCount: favoriteCount,
      columnIsNew: isNew ? 1 : 0,
      columnIsAvailable: isAvailable == null
          ? null
          : isAvailable
              ? 1
              : 0,
      columnLastTimestamp: lastTimestamp,
    };
    return map;
  }

  void fromMap(Map<String, dynamic> map) {
    id = map[columnId];
    name = map[columnName];
    code = map[columnCode];
    auxCode = map[columnAuxCode];
    referenceCode = map[columnReferenceCode];
    description = map[columnDescription];
    additionalInfo = map[columnAdditionalInfo];
    imageUrl = map[columnImageUrl];

    ///OLD PRICE PROPERTIES
    ecommerceId = map[columnEcommerceID];
    ecommercePrice = map[columnEcommercePrice]?.toDouble();
    ecommerceStock = map[columnEcommercestock];
    kioskId = map[columnKioskId];
    kioskPrice = map[columnKioskPrice]?.toDouble();
    kioskStock = map[columnKioskstock];
    defaultPrice = map[columnDefaultPrice]?.toDouble();
    singleOldPrice = map[columnSingleOldPrice]?.toDouble();
    singlePrice = map[columnSinglePrice]?.toDouble();
    maxKioskInstallments = map[columnMaxKioskInstallments];
    kioskInstallmentPrice = map[columnKioskInstallmentPrice]?.toDouble();
    maxEcommerceInstallments = map[columnMaxEcommerceInstallments];
    ecommerceInstallmentPrice = map[columnEcommerceInstallmentPrice]?.toDouble();
    maxSingleInstallments = map[columnMaxsingleInstallments];
    singleInstallmentPrice = map[columnSingleInstallmentPrice]?.toDouble();
    isSinglePrice = map[columnIssinglePrice] != null && map[columnIssinglePrice] == 1 ? true : false;

    ///OLD PRICE PROPERTIES
    ///
    ///NEW PRICE PROPERTIES
    price = map[columnPrice]?.toDouble();
    oldPrice = map[columnOldPrice]?.toDouble();
    maxInstallments = map[columnMaxInstallments]?.toInt();
    installmentPrice = map[columnInstallmentPrice]?.toDouble();

    ///NEW PRICE PROPERTIES
    isFavorite = map[columnIsFavorite] != null && map[columnIsFavorite] == 1 ? true : false;
    favoriteCount = map[columnFavoriteCount];
    isNew = map[columnIsNew] != null && map[columnIsNew] == 1 ? true : false;
    isAvailable = map[columnIsAvailable] != null && map[columnIsAvailable] == 1
        ? true
        : false;

    List<FeaturedCharacteristic> featuredCharacteristicItems = [];
    if(map[columnFeaturedCharacteristics] != null){
      Iterable iItems = json.decode(map[columnFeaturedCharacteristics]);
      iItems.forEach((map) {
        FeaturedCharacteristic item = FeaturedCharacteristic.fromJson(map);
        featuredCharacteristicItems.add(item);
      });
    }
    this.featuredCharacteristics = featuredCharacteristicItems;
    lastTimestamp = map[columnLastTimestamp];
  }
}

@JsonSerializable()
class Sku {
  Sku();

  String id;
  String code;
  String name;
  String imageUrl;
  String selectionImageUrl;
  String color;

  String productSku;
  String productCode;
  Product product;

  List<ProductImage> images;
  List<SkuSpecificAttribute> attributes;

  @JsonKey(defaultValue: true)
  bool isAvailable;

  String correctName() {
    return name == null ? code : name;
  }

  factory Sku.fromJson(Map<String, dynamic> json) => _$SkuFromJson(json);

  Map<String, dynamic> toJson() => _$SkuToJson(this);
}

extension KeyValueHtml on Iterable<KeyValue> {
  String get htmlString {
    if (this.length == 0) return '';

    var keyValueHtml = '';

    for (var characteristic in this) {
      var charHtml = '<p>';
      charHtml += '<b><span class="c-title">${characteristic.key}</span></b>';
      charHtml += '</br>';
      charHtml += '<span class="c-body">${characteristic.value}</span>';
      charHtml += '</p>';

      keyValueHtml += charHtml;
    }

    keyValueHtml = keyValueHtml +
        '''
      <style>
      .c-title {
        color: blue;
        text-align: left;
        font-weight: bold;
        width: 100%;
      }
      
      .c-body {
        background-color: red;
        text-align: left;
        width: 100%;
      }
      </style>
    ''';

    return keyValueHtml;
  }
}

==> lori_sdk-0.9.2.tar.gz: lib/data/models/product/product.dart <==
import 'dart:convert';

import 'package:lori_sdk/data/lori_api.dart';
import 'package:lori_sdk/data/models/product/featuredCharacteristic.dart';
import 'package:lori_sdk/data/models/product/productImage.dart';
import 'package:lori_sdk/data/models/product/skuAttribute.dart';
import 'package:lori_sdk/data/models/key_value.dart';

part 'product.g.dart';

const ORDER_RELEVANCE = "Relevance";
const ORDER_HIGHER = "HigherPrice";
const ORDER_LOWEST = "LowestPrice";
const ORDER_FAVORED = "MostFavored";
const ORDER_SELLERS = "BestSellers";

final String tableName = 'product';
final String columnId = 'id';
final String columnName = 'name';
final String columnCode = 'code';
final String columnAuxCode = 'aux_code';
final String columnReferenceCode = 'reference_code';
final String columnDescription = 'description';
final String columnAdditionalInfo = 'additional_info';
final String columnImageUrl = 'image_url';

///OLD PRICE PROPERTIES
final String columnEcommerceID = 'ecommerce_id';
final String columnEcommercePrice = 'ecommerce_price';
final String columnEcommercestock = 'ecommerce_stock';
final String columnKioskId = 'kiosk_id';
final String columnKioskPrice = 'kiosk_price';
final String columnKioskstock = 'kiosk_stock';
final String columnDefaultPrice = 'default_price';
final String columnSingleOldPrice = 'single_old_price';
final String columnSinglePrice = 'single_price';
final String columnMaxKioskInstallments = 'max_kiosk_installments';
final String columnKioskInstallmentPrice = 'kiosk_installment_price';
final String columnMaxEcommerceInstallments = 'max_ecommerce_installments';
final String columnEcommerceInstallmentPrice = 'ecommerce_installment_price';
final String columnMaxsingleInstallments = 'max_single_installments';
final String columnSingleInstallmentPrice = 'single_installment_price';
final String columnIssinglePrice = 'is_single_price';
final String columnFeaturedCharacteristics = 'featured_characteristics';

///OLD PRICE PROPERTIES

///NEW PRICE PROPERTIES
final String columnPrice = 'price';
final String columnOldPrice = 'old_price';
final String columnMaxInstallments = 'max_installments';
final String columnInstallmentPrice = 'installment_price';

///OLD PRICE PROPERTIES

final String columnIsFavorite = 'is_favorite';
final String columnFavoriteCount = 'favorite_count';
final String columnIsNew = 'is_new';
final String columnIsAvailable = 'is_available';
final String columnLastTimestamp = 'lastTimestamp';

class Product {
  Product();

  String id;
  String name;
  String code;
  String auxCode;
  String complementaryName;
  String referenceCode;
  String description;
  String additionalInfo;
  String imageUrl;
  String additionalInformation;

  ///
  /// region [START] Prices variables for LEGACY API
  ///

  String ecommerceId;
  double ecommercePrice;
  int ecommerceStock;

  String kioskId;
  double kioskPrice;
  int kioskStock;

  double defaultPrice;
  double singleOldPrice;
  double singlePrice;
  double unitPrice;

  int maxKioskInstallments;
  double kioskInstallmentPrice;
  int maxEcommerceInstallments;
  double ecommerceInstallmentPrice;
  int maxSingleInstallments;
  double singleInstallmentPrice;
  bool isSinglePrice;

  ///
  /// [END] Prices variables for LEGACY API
  /// endregion
  ///

  ///
  /// region [START] Prices properties for MODERN API
  ///
  ///

  double price;
  double oldPrice;
  double wholesalePrice;
  int wholesaleMinQuantity;

  int maxInstallments;
  double installmentPrice;

  ///
  /// [END] Prices properties for MODERN API

  List<Sku> skus;
  List<ProductImage> images;

  bool isFavorite;
  int favoriteCount;
  bool isNew;

  bool isAvailable;

  int lastTimestamp;

  bool pickupOnly;

  List<FeaturedCharacteristic> featuredCharacteristics;

  List<KeyValue> characteristics;

  List<KeyValue> orderCharacteristics() {
    if (this.characteristics != null && this.characteristics.isNotEmpty) {
      this.characteristics.sort((a, b) => a.order.compareTo(b.order));
      return this.characteristics;
    }
    return this.characteristics;
  }

  bool haveMessagefeaturedCharacteristics() {
    if (featuredCharacteristics != null && featuredCharacteristics.isNotEmpty) {
      return true;
    }
    return false;
  }

  double getOldPrice() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.oldPrice != null)
      return this.oldPrice;

    // Fallback to legacy API
    if (isSinglePrice) {
      return singleOldPrice;
    } else {
      return singleOldPrice;
    }
  }

  double getProductPrice() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.price != null && this.price > 0)
      return this.price;

    // Fallback to legacy API
    if (isSinglePrice == true)
      return singlePrice;
    else if (this.kioskPrice == null && this.ecommercePrice == null) {
      return .0;
    } else {
      if (this?.kioskPrice <= this?.ecommercePrice)
        return this?.kioskPrice;
      else
        return this?.ecommercePrice;
    }
  }

  double getInstallmentPrice() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.installmentPrice != null)
      return this.installmentPrice;

    // Fallback to legacy API
    if (isSinglePrice)
      return this.singleInstallmentPrice;
    else {
      if (this.kioskInstallmentPrice <= this.ecommerceInstallmentPrice)
        return this.kioskInstallmentPrice;
      else
        return this.ecommerceInstallmentPrice;
    }
  }

  int getMaxInstallments() {
    // Always try to use the new properties
    if (!LoriApi.useLegacyCatalogApi && this.maxInstallments != null)
      return this.maxInstallments;

    // Fallback to legacy API
    if (isSinglePrice)
      return this.maxSingleInstallments;
    else {
      if (this.maxKioskInstallments <= this.maxEcommerceInstallments)
        return this.maxKioskInstallments;
      else
        return this.maxEcommerceInstallments;
    }
  }

  String displayCode({String displayCodeName}) {
    switch (displayCodeName.toLowerCase()) {
      case "referencecode":
        //Reference Code is NOT mandatory, so we must have a fallback
        if (this.referenceCode == null) return this.code;
        return this.referenceCode.isNotEmpty ? this.referenceCode : this.code;
      case "auxcode":
        //Aux Code is NOT mandatory, so we must have a fallback
        if (this.auxCode == null) return this.code;
        return this.auxCode.isNotEmpty ? this.auxCode : this.code;
      case "code":
        return this.code;
      default:
        return this.code;
    }
  }

  List<String> get tags {
    var tags = <String>[];

    if (this.getOldPrice() != null &&
        this.getOldPrice() > this.getProductPrice()) {
      // We have a discount at this product
      // Get delta value
      var discountValue = this.getOldPrice() - this.getProductPrice();
      // Get percentage in decimal (i.e., 0,15)
      var decimalPercentage = discountValue / this.getOldPrice();
      // Multiply by 100 to get the absolute percentage (i.e., 15)
      var percentage = decimalPercentage * 100;
      if (percentage >= 9.0 && percentage <= 10.0) {
        // tags = 9;
        tags.add(percentage.round().toString() + "% OFF");
        return tags;
      } else {
        tags.add(percentage
                .round()
                .toString()
                .substring(0, percentage.toString().indexOf('.')) +
            "% OFF");
      }
      return tags;
      // if(percentage is int || percentage == percentage.roundToDouble()){
      //   tags.add(percentage.toStringAsFixed(0) + "% OFF");
      // } else {
      //   percentage = (percentage * 100).truncateToDouble() / 100;
      //   tags.add(percentage.toString() + "% OFF");
      // }
    }

    return tags;
  }

  factory Product.fromJson(Map<String, dynamic> json) {
    var product = _$ProductFromJson(json);
    if (product.skus != null) {
      // Sort the skus alphabetically
      product.skus.sort((ls, rs) => ls.name.compareTo(rs.name));
    }

    return product;
  }

  Map<String, dynamic> toJson() => _$ProductToJson(this);

  Map<String, dynamic> toMap() {
    var map = <String, dynamic>{
      columnId: id,
      columnName: name,
      columnCode: code,
      columnAuxCode: auxCode,
      columnReferenceCode: referenceCode,
      columnDescription: description,
      columnAdditionalInfo: additionalInfo,
      columnImageUrl: imageUrl,

      ///OLD PRICE PROPERTIES
      columnEcommerceID: ecommerceId,
      columnEcommercePrice: ecommercePrice?.toDouble(),
      columnEcommercestock: ecommerceStock,
      columnKioskId: kioskId,
      columnKioskPrice: kioskPrice?.toDouble(),
      columnKioskstock: kioskStock,
      columnDefaultPrice: defaultPrice?.toDouble(),
      columnSingleOldPrice: singleOldPrice?.toDouble(),
      columnSinglePrice: singlePrice?.toDouble(),
      columnMaxKioskInstallments: maxKioskInstallments,
      columnKioskInstallmentPrice: kioskInstallmentPrice?.toDouble(),
      columnMaxEcommerceInstallments: maxEcommerceInstallments,
      columnEcommerceInstallmentPrice: ecommerceInstallmentPrice?.toDouble(),
      columnMaxsingleInstallments: maxSingleInstallments,
      columnSingleInstallmentPrice: singleInstallmentPrice?.toDouble(),
      columnIssinglePrice: isSinglePrice == null
          ? null
          : isSinglePrice
              ? 1
              : 0,
      columnFeaturedCharacteristics: json.encode(featuredCharacteristics),

      ///OLD PRICE PROPERTIES
      ///
      ///NEW PRICE PROPERTIES
      columnPrice: price,
      columnOldPrice: oldPrice,
      columnMaxInstallments: maxInstallments,
      columnInstallmentPrice: installmentPrice,

      ///NEW PRICE PROPERTIES

      columnIsFavorite: isFavorite ? 1 : 0,
      columnFavoriteCount: favoriteCount,
      columnIsNew: isNew ? 1 : 0,
      columnIsAvailable: isAvailable == null
          ? null
          : isAvailable
              ? 1
              : 0,
      columnLastTimestamp: lastTimestamp,
    };
    return map;
  }

  void fromMap(Map<String, dynamic> map) {
    id = map[columnId];
    name = map[columnName];
    code = map[columnCode];
    auxCode = map[columnAuxCode];
    referenceCode = map[columnReferenceCode];
    description = map[columnDescription];
    additionalInfo = map[columnAdditionalInfo];
    imageUrl = map[columnImageUrl];

    ///OLD PRICE PROPERTIES
    ecommerceId = map[columnEcommerceID];
    ecommercePrice = map[columnEcommercePrice]?.toDouble();
    ecommerceStock = map[columnEcommercestock];
    kioskId = map[columnKioskId];
    kioskPrice = map[columnKioskPrice]?.toDouble();
    kioskStock = map[columnKioskstock];
    defaultPrice = map[columnDefaultPrice]?.toDouble();
    singleOldPrice = map[columnSingleOldPrice]?.toDouble();
    singlePrice = map[columnSinglePrice]?.toDouble();
    maxKioskInstallments = map[columnMaxKioskInstallments];
    kioskInstallmentPrice = map[columnKioskInstallmentPrice]?.toDouble();
    maxEcommerceInstallments = map[columnMaxEcommerceInstallments];
    ecommerceInstallmentPrice =
        map[columnEcommerceInstallmentPrice]?.toDouble();
    maxSingleInstallments = map[columnMaxsingleInstallments];
    singleInstallmentPrice = map[columnSingleInstallmentPrice]?.toDouble();
    isSinglePrice =
        map[columnIssinglePrice] != null && map[columnIssinglePrice] == 1
            ? true
            : false;

    ///OLD PRICE PROPERTIES
    ///
    ///NEW PRICE PROPERTIES
    price = map[columnPrice]?.toDouble();
    oldPrice = map[columnOldPrice]?.toDouble();
    maxInstallments = map[columnMaxInstallments]?.toInt();
    installmentPrice = map[columnInstallmentPrice]?.toDouble();

    ///NEW PRICE PROPERTIES
    isFavorite = map[columnIsFavorite] != null && map[columnIsFavorite] == 1
        ? true
        : false;
    favoriteCount = map[columnFavoriteCount];
    isNew = map[columnIsNew] != null && map[columnIsNew] == 1 ? true : false;
    isAvailable = map[columnIsAvailable] != null && map[columnIsAvailable] == 1
        ? true
        : false;

    List<FeaturedCharacteristic> featuredCharacteristicItems = [];
    if (map[columnFeaturedCharacteristics] != null) {
      Iterable iItems = json.decode(map[columnFeaturedCharacteristics]);
      iItems.forEach((map) {
        FeaturedCharacteristic item = FeaturedCharacteristic.fromJson(map);
        featuredCharacteristicItems.add(item);
      });
    }
    this.featuredCharacteristics = featuredCharacteristicItems;
    lastTimestamp = map[columnLastTimestamp];
  }
}

class Sku {
  Sku();

  String id;
  String code;
  String name;
  String imageUrl;
  String selectionImageUrl;
  String color;

  String productSku;
  String productCode;
  Product product;

  List<ProductImage> images;
  List<SkuSpecificAttribute> attributes;

  bool isAvailable;

  String correctName() {
    return name == null ? code : name;
  }

  factory Sku.fromJson(Map<String, dynamic> json) => _$SkuFromJson(json);

  Map<String, dynamic> toJson() => _$SkuToJson(this);
}

extension KeyValueHtml on Iterable<KeyValue> {
  String get htmlString {
    if (this.length == 0) return '';

    var keyValueHtml = '';

    for (var characteristic in this) {
      var charHtml = '<p>';
      charHtml += '<b><span class="c-title">${characteristic.key}</span></b>';
      charHtml += '</br>';
      charHtml += '<span class="c-body">${characteristic.value}</span>';
      charHtml += '</p>';

      keyValueHtml += charHtml;
    }

    keyValueHtml = keyValueHtml +
        '''
      <style>
      .c-title {
        color: blue;
        text-align: left;
        font-weight: bold;
        width: 100%;
      }
      
      .c-body {
        background-color: red;
        text-align: left;
        width: 100%;
      }
      </style>
    ''';

    return keyValueHtml;
  }
}
